# Ruby ConjugateFR Renderers - Part of Ruby ConjugateFR.
# I, htmlguy, hereby release this work into the public domain.

# Renders conjugated french verbs.
# (see conjugateFR)
class Renderer
  # Prints content to go before the conjugations.
  def pre; end
  # Prints a verb and it's conjugations.
  # @param name [String] The verb.
  # @param words [Array] An array of conjugations.
  def word (name, words); end
  # Prints content to go after the conjugations.
  def post; end
  # The description of this renderer.
  # For example: +Renders in the example format.+
  # @return [String] the description.
  def description; ""; end
end

# A renderer in a human-readable format.
# Output looks like:
#   == someverb ==
#  someconjugation
#  someconjugation
#  etc...
class HumanRenderer < Renderer
  def word (name, words)
    puts "\n == #{name} == "
    words.each do |val|
      puts val
    end
  end
  def description; "Renders in a nice, human-readable format."; end
end

# A renderer template for rendering in a delimitered table format.
# (see CSVRenderer)
# (see TSVRenderer)
class DelimitedTableRenderer < Renderer
  def initialize; @s = ""; end
  def pre
    puts "Word#{@s}Je#{@s}Tu#{@s}Il/Elle/On#{@s}Nous#{@s}Vous#{@s}Ils/Elles"
  end
  def word (name, words)
    print "#{name}"
    words.each do |word|
      print "#{@s}#{word}"
    end
    puts
  end
end

# A renderer in the comma-separated value format.
# Output looks like:
#  Word,Je,Tu,Il/Elle/On,Nous,Vous,Ils/Elles
#  someverb,conj1,conj2,conj3,conj4,conj5,conj6
class CSVRenderer < DelimitedTableRenderer
  def initialize; @s = ","; end
  def description; "Renders in the Comma Separated Value (CSV) format."; end
end

# A renderer in the tab-separated value format.
# Output looks like (w/ 4-space tab):
#  Word    Je  Tu  Il/Elle/On  Nous    Vous    Ils/Elles
#  someverb    conj1   conj2   conj3   conj4   conj5   conj6
class TSVRenderer < DelimitedTableRenderer
  def initialize; @s = "\t"; end
  def description; "Renders in the Tab Separated Value (TSV) format."; end
end

# A renderer in the markdown table syntax.
# Output looks like:
#  |Word|Je|Tu|Il/Elle/On|Nous|Vous|Ils/Elles
#  |---|---|---|---|---|---|---
#  |someverb|conj1|conj2|conj3|conj4|conj5|conj6
class MDTableRenderer < Renderer
  def pre
    puts "|Word|Je|Tu|Il/Elle/On|Nous|Vous|Ils/Elles"
    puts "|---|---|---|---|---|---|---"
  end
  def word (name, words)
    print "|#{name}"
    words.each do |word|
      print "|#{word}"
    end
    puts
  end
  def description; "Renders in Markdown's table syntax."; end
end

# Renders verbs in the form of a count of how many there are.
class CounterRenderer < Renderer
  def initialize; @count = 0; end
  def word (name, words)
    @count += 1
  end
  def post
    puts @count
  end
  def description; "Renders out a count of how many words are listed."; end
end

# A global list of renderers, with each one assigned to it's human-readable
# name.
# Used for programs like the CLI.
$renderers = {
  "Human" => HumanRenderer.new,
  "CSV" => CSVRenderer.new,
  "TSV" => TSVRenderer.new,
  "MDTable" => MDTableRenderer.new,
  "Counter" => CounterRenderer.new
}
